package jp.flashcast.translator.android.service;

import java.util.Date;
import java.util.List;

import jp.flashcast.translator.android.R;
import jp.flashcast.translator.android.dao.TranslateDao;
import jp.flashcast.translator.android.model.TranslateModel;
import jp.flashcast.translator.android.thread.TranslateThread;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.text.ClipboardManager;

public class TranslateService extends Service implements SensorEventListener, Runnable {
	private SensorManager manager;
	private boolean registered;
	private boolean binded;
	private TranslateDao dao;
	private TranslateModel model;
	private float[] accelerations = {0.0f, 0.0f, 0.0f};
	
	public class TranslateBinder extends Binder {
		public TranslateService getService() {
			return TranslateService.this;
		}
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		binded = true;
		return new TranslateBinder();
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		
		manager = (SensorManager)getApplicationContext().getSystemService(SENSOR_SERVICE);
		List<Sensor> sensors = manager.getSensorList(Sensor.TYPE_ACCELEROMETER);
		
		if (sensors.size() > 0) {
			Sensor sensor = sensors.get(0);
			registered = manager.registerListener((SensorEventListener)this, sensor, SensorManager.SENSOR_DELAY_NORMAL);
		}
		
        if (dao == null) {
        	dao = new TranslateDao(this);
        }
	}
	
	public void onAccuracyChanged(Sensor sensor, int accuracy) {
		
	}

	public void onSensorChanged(SensorEvent event) {
		if (!binded) {
			if (event.sensor.getType() == Sensor.TYPE_ACCELEROMETER) {
				accelerations[0] = event.values[0] - accelerations[0];
				accelerations[1] = event.values[1] - accelerations[1];
				accelerations[2] = event.values[2] - accelerations[2];
				
				float acceleration = 
					Math.abs(accelerations[0]) + 
					Math.abs(accelerations[1]) +
					Math.abs(accelerations[2]);
				
				if (acceleration > 40.0f) {
					String clipboard =
						((ClipboardManager)(getApplicationContext().getSystemService(CLIPBOARD_SERVICE)))
							.getText().toString();
					
					if (!clipboard.equals("")) {
						// 翻訳実行
						if (model == null) {
							
						}
						model = new TranslateModel();
						model.setOriginal(clipboard);
						model.setSrcLanguage("en");
						model.setDstLanguage("ja");
						model.setDt(new Date().toLocaleString());

						Translate();
					}
					
					accelerations[0] = 0.0f;
					accelerations[1] = 0.0f;
					accelerations[2] = 0.0f;
				}
			}
		}
	}
	
	private void Translate() {
		new TranslateThread(new Handler(), this, model).start();
	}
	
	@Override
	public void onDestroy() {
		if (registered) {
			manager.unregisterListener(this);
			registered = false;
		}
		
		super.onDestroy();
	}
	
	@Override
	public void onRebind(Intent intent) {
		binded = true;
	}
	
	@Override
	public boolean onUnbind(Intent intent) {
		binded = false;
		
		return true; // 再度クライアントから接続された際に onRebind を呼び出させる場合は true を返す
	}
	
	public void run() {
		if (model.isSuccess()) {
			dao.insert(model);
			
			NotificationManager manager =
				(NotificationManager)(getApplicationContext().getSystemService(NOTIFICATION_SERVICE));
			
			PendingIntent intent = PendingIntent.getActivity(this, 0, null, Intent.FLAG_ACTIVITY_NEW_TASK);
			Notification notification = new Notification(android.R.drawable.ic_menu_info_details, getText(R.string.app_name), System.currentTimeMillis());
			notification.setLatestEventInfo(getApplicationContext(), getText(R.string.app_name), "translated!", intent);
			notification.flags = Notification.FLAG_AUTO_CANCEL;
			manager.notify(R.string.app_name, notification);
		}
	}	
}